/**
  ******************************************************************************
  * @file    TIM4/TIM4_TimeBase/main.c
  * @author  MCD Application Team
  * @version V1.5.2
  * @date    30-September-2014
  * @brief   Main program body
  ******************************************************************************
  */ 

// Includes ------------------------------------------------------------------
#include "stm8l15x.h"
#include "stm8_eval.h"

 // @addtogroup STM8L15x_StdPeriph_Examples

 // @addtogroup TIM4_TimeBase

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
#define TIM4_PERIOD       124

// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
__IO uint32_t TimingDelay;

// Private function prototypes -----------------------------------------------
static void CLK_Config(void);
static void TIM4_Config(void);
void Delay(__IO uint32_t nTime);
void TimingDelay_Decrement(void);
// Private functions ---------------------------------------------------------

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
void main(void)
{
   // CLK configuration -------------------------------------------
  CLK_Config();

  // Initialize Leds mounted on STM8L1526-EVAL board 
  STM_EVAL_LEDInit(LED1);
  STM_EVAL_LEDInit(LED2);
  STM_EVAL_LEDInit(LED3);
  STM_EVAL_LEDInit(LED4);

  // Turn on LED1 and LED3 
  STM_EVAL_LEDOn(LED1);
  STM_EVAL_LEDOn(LED3);

   // TIM4 configuration -------------------------------------------
  TIM4_Config();
  
  while (1)
  {
    // Toggle LED2 and LED4 
    STM_EVAL_LEDToggle(LED2);
    STM_EVAL_LEDToggle(LED4);

    // Insert 50 ms delay 
    Delay(50);

    // Toggle LED1 and LED3 
    STM_EVAL_LEDToggle(LED1);
    STM_EVAL_LEDToggle(LED3);

    // Insert 100 ms delay 
    Delay(100);
  }
}

/**
  * @brief  Configure peripherals Clock   
  * @param  None
  * @retval None
  */
static void CLK_Config(void)
{
  // High speed internal clock prescaler: 1 
  CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);

  // Select HSE as system clock source 
  CLK_SYSCLKSourceSwitchCmd(ENABLE);
  CLK_SYSCLKSourceConfig(CLK_SYSCLKSource_HSE);

  while (CLK_GetSYSCLKSource() != CLK_SYSCLKSource_HSE)
  {}

  // Enable TIM4 CLK 
  CLK_PeripheralClockConfig(CLK_Peripheral_TIM4, ENABLE);
}

/**
  * @brief  Configure TIM4 peripheral   
  * @param  None
  * @retval None
  */
static void TIM4_Config(void)
{
  /* TIM4 configuration:
   - TIM4CLK is set to 16 MHz, the TIM4 Prescaler is equal to 128 so the TIM1 counter
   clock used is 16 MHz / 128 = 125 000 Hz
  - With 125 000 Hz we can generate time base:
      max time base is 2.048 ms if TIM4_PERIOD = 255 --> (255 + 1) / 125000 = 2.048 ms
      min time base is 0.016 ms if TIM4_PERIOD = 1   --> (  1 + 1) / 125000 = 0.016 ms
  - In this example we need to generate a time base equal to 1 ms
   so TIM4_PERIOD = (0.001 * 125000 - 1) = 124 */

  // Time base configuration 
  TIM4_TimeBaseInit(TIM4_Prescaler_128, TIM4_PERIOD);
  // Clear TIM4 update flag 
  TIM4_ClearFlag(TIM4_FLAG_Update);
  // Enable update interrupt 
  TIM4_ITConfig(TIM4_IT_Update, ENABLE);
  // enable interrupts 
  enableInterrupts();

  // Enable TIM4 
  TIM4_Cmd(ENABLE);
}
/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in milliseconds.
  * @retval None
  */
void Delay(__IO uint32_t nTime)
{
  TimingDelay = nTime;

  while (TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  {
    TimingDelay--;
  }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  // Infinite loop 
  while (1)
  {}
}
#endif



